function cantilever(filename)
%cantilever(filename)
%
%  Description:
%
%    This is the Octave/MATLAB implimentation of the cantilever model
%    that is used in the Dakota training materials.
%
%  Inputs:
%
%    filename = the name of the input file
%
%  Outputs:
%
%    The results are written to: cantilever.o
%
%  Contact:
%
%    Bob Cochran
%    Applied CHT
%    rjc@heattransfer.org
%
%  History:
%
%    Who    Date   Version  Note
%    ---  -------- -------  -----------------------------------------------
%    RJC  05/16/17  0.1.0   Initial release.
%
%==========================================================================

% Math model parameters to be parsed from the input file

thickness       = NaN;
width           = NaN;
length          = NaN;
density         = NaN;
Youngs_modulus  = NaN;
vertical_load   = NaN;
horizontal_load = NaN;

%--------------------------------------------------------------------------
% Open and parse the input file
%--------------------------------------------------------------------------

fid = fopen(filename);
while 1
  line = fgetl(fid);  %  Fetch the next line from the input file
  if ~ischar(line)    %  Check for the end of file
    if ~fclose(fid)   %  Close the file      
      break
    else
      error('Error closing the data file.')
    end
  end
  str = regexp(line, '#', 'split');  %  str{1} is line before a comment
  str = strtrim(str{1});  %  Trim off the leading/trailing blanks
  if ~isempty(str)
    tok = regexp(str,'[a-zA-Z_0-9+-./]*','match');  %  Split string into tokens
    if strcmpi(tok{1},'thickness') 
      thickness = str2double(tok{2});
    elseif strcmpi(tok{1},'width') 
      width = str2double(tok{2});
    elseif strcmpi(tok{1},'length') 
      length = str2double(tok{2});
    elseif strcmpi(tok{1},'density')
      density = str2double(tok{2});
    elseif strcmpi(tok{1},'Youngs_modulus')
      Youngs_modulus = str2double(tok{2});
    elseif strcmpi(tok{1},'vertical_load') 
      vertical_load = str2double(tok{2});
    elseif strcmpi(tok{1},'horizontal_load')
      horizontal_load = str2double(tok{2});
    end
  end
end

%--------------------------------------------------------------------------
% Compute the solution to the math model
%--------------------------------------------------------------------------

area = thickness*width;

mass = (area*density*length)/(12.0^3);

stress = (600.0/(area*thickness))*vertical_load +   ...
         (600.0/(area*width))*horizontal_load;
       
displacement = ((4.0*length^3)/(area*Youngs_modulus))*      ...
               sqrt((vertical_load/(thickness^2.0))^2 +     ...
                    (horizontal_load/(width^2))^2);

%--------------------------------------------------------------------------
% Write the solution to the output file
%--------------------------------------------------------------------------

fid = fopen('cantilever.out','w');
fprintf(fid,' %15.8e \n', mass);
fprintf(fid,' %15.8e \n', stress);
fprintf(fid,' %15.8e ', displacement);
fclose(fid);
